/* Application graphique en utilisant une carte mémoire SD.
   Une image monochrome au format BMP étant dans un fichier nommé
   IMGtest.BMP le programme effectue une analyse de cette dernière,
   affiche ses caractéristiques sur la ligne série.
   Puis il crée un fichier TABLEAU.txt et y transcrit en binaire
   les octets de l'image avec des séparations pour que le texte
   produit soit directement compatible avec la création d'un
   tableau permettant d'utiliser cette image comme LOGO dans un
   programme quelconque pour ARDUINO.                            */

#include <SPI.h>

#include <SD.h> // Bibliothèque présente par défaut dans l'IDE.

const byte ChipSelect = 10;  //Broche 10 reliée au CS du module SD.
Sd2Card Carte_SSD;
SdVolume Contenu;
SdFile Racine;
File Mon_Fichier, Fichier_tableau;

int Debut, Taille;

void setup() {
  Serial.begin(115200);  while (!Serial) { ; }
  pinMode(10, OUTPUT);
  // Vérification de la présence carte.
  if (!Carte_SSD.init(SPI_HALF_SPEED, ChipSelect))
     { Serial.println("\nPas de carte dans le lecteur."); return; }
  // Vérification des données.
  if (!Contenu.init(Carte_SSD)) return;
  // Afficher la liste des fichiers présents.
  Serial.println("\nFichiers actuels sur la carte :");
  Serial.println("Nom             Date      Heure");
  Racine.openRoot(Contenu);  
  Racine.ls(LS_R | LS_DATE | LS_SIZE);
  // On se prepare a intervenir dans les fichiers.
  if (!SD.begin(ChipSelect)) { Serial.println("\nEchec de l'initialisation."); return; }
  // Ouverture du fichier pour lecture.
  Mon_Fichier = SD.open("IMGtest.bmp");
  if (!Mon_Fichier) Serial.println("\nImpossible d'ouvrir le fichier pour sa lecture");
  else {Serial.println("\nContenu du fichier IMGtest.bmp :"); Serial.println();   
     // Lire et afficher les premiers octets.
     Serial.print("Type du fichier : "); Serial.print(char(Mon_Fichier.read()));
     Serial.println(char(Mon_Fichier.read()));
     Serial.print("Taille du fichier : "); Serial.println(Lire_un_Long());
     Mon_Fichier.seek(18);
     Serial.print("Largeur de l'image : "); Serial.println(Lire_un_Long());
     Serial.print("Hauteur de l'image : "); Serial.println(Lire_un_Long());
     Serial.print("Nombre de plans : "); Serial.println(Lire_un_Entier());
     Serial.print("Nombre de couleurs : "); Serial.println(Lire_un_Entier());
     Serial.print("Methode de compression : "); Serial.println(Lire_un_Long());
     Taille = Lire_un_Long();
     Serial.print("Taille totale de l'image : "); Serial.println(Taille);
     Serial.print("Resolution H : "); Serial.println(Lire_un_Long());
     Serial.print("Resolution V : "); Serial.println(Lire_un_Long());
     Serial.print("Num couleur palette : "); Serial.println(Lire_un_Long());
     Serial.print("Nb couleurs importantes : "); Serial.println(Lire_un_Long());
     Mon_Fichier.seek(10); Debut = Lire_un_Long();    
     Serial.print("Debut de l'image en : "); Serial.println(Debut);
     Serial.print("taille en-tete : "); Serial.println(Lire_un_Long());
     
  Serial.println("\nRecuperation de l'Image.");
  Serial.println("PATIENCE, Il faut 20 secondes pour 600 Octets."); 
  // Effacement du fichier TABLEAU.txt pour le vider si il existe.
  SD.remove("TABLEAU.txt"); 
  // Ouverture du fichier pour ecriture (Il sera créé s'il n'existait pas.
  Fichier_tableau = SD.open("TABLEAU.txt", FILE_WRITE);
  Recupere_image_dans_TABLEAUtxt();}
  Serial.println("\nTaille du fichier TABLEAU.txt");
  Taille = Fichier_tableau.size(); Serial.println(Taille);
  Affiche_TABLEAUtxt();
  Mon_Fichier.close(); Fichier_tableau.close();}  // Fermeture des deux fichiers.

void loop(void) {  }

void Affiche_TABLEAUtxt() {
  int PTR; 
  Serial.println("\nContenu du fichier TABLEAU.txt :");
  Fichier_tableau.seek(0); PTR = 0;
  while (PTR < Taille) {Serial.print(char(Fichier_tableau.read())); PTR++;}  }
       
void Recupere_image_dans_TABLEAUtxt() {
  /*************** récupération de l'image dans TABLEAU.txt ***************/
  // Tous les 16 octets on génère un passage à la ligne.  
  byte Compteur; Compteur = 0;
  Mon_Fichier.seek(Debut); 
  for (int PTR = 0; PTR < Taille; PTR++) {
    Fichier_tableau.print(Mon_Fichier.read());
    Fichier_tableau.write(", "); Compteur++;
    if (Compteur == 16) {Compteur = 0; Fichier_tableau.println();} } }

long Lire_un_Long() {// Lit quatre octets dans le fichier et calcule la valeur.
  long VALEUR; VALEUR = Mon_Fichier.read();
  for (byte I=1; I<4; I++) VALEUR = VALEUR + (Mon_Fichier.read()* 256 * I);
  return VALEUR;}
  
int Lire_un_Entier() {// Lit deux octets dans le fichier et calcule la valeur.
  int VALEUR; VALEUR = Mon_Fichier.read();
  VALEUR = VALEUR + (Mon_Fichier.read()* 256);
  return VALEUR;}
