/*  NANO station MÉTÉO avec affichage sur un module OLED SSD1306.
           Par Michel DROUI. (Pseudo Internet : Nulentout.)
           
    Version actuelle : 25300 octets de programme (82%) et 822 octets (40%) de mémoire dynamique.
                     (Mais la mémoire dynamique durant le programme s'encombre des échantillons.)

    Programme strictement identique à P10 mais légèrement modifié pour afficher la grenouille.
    la taille diminue un peu car on ne complète plus le dessin avec deux lignes comme c'était
    le car pour tracer la Salamandre. 
    Le textes et le dessin ont été recadrés pour meilleur centrage.                                   */
    
// Déclarations pour l'afficheur OLED :
#include <Adafruit_ssd1306syp.h>
#define broche_SDA 11 // D11 utilisée en sortie.
#define broche_SCL 12 // D12 utilisée en sortie.
Adafruit_ssd1306syp display(broche_SDA, broche_SCL);

// Déclarations pour le capteur codeur rotatif KY-40 :
#define Codeur_A 3   // D3 : INT 1 sur broche 5 du microcontrôleur. (CLK)
#define Codeur_B 2    // D2 : INT 0 sur broche 4 du microcontrôleur. (DT)
#define BP_central 15 // A1 pour lire le bouton central.

// Déclarations pour le capteur de pression BMP085.
// Mesure la pression de 300hPa à 1100hPa (soit de 9000m d’altitude à 500 sous le niveau de la mer)
// Lecture de la température entre -40 et 85°C (précision maximale entre 0 et 60°C)
// Résolution de 0.01 hPa et 0.1°C
// Précision de 0.2 à 0.5 hPa et de 0.5 à 1°C (en fonction de la température)
#include <Wire.h>
#include <Adafruit_BMP085.h> // Utilisation du capteur de pression atmosphérique BMP085.
  // BRANCHEMENTS imposés par la bibliothèque :
  // SLC est reliée à  l'entrée Analogique A5.
  // SDA est reliée à  l'entrée Analogique A4.
Adafruit_BMP085 Capteur_BMP085;

// Déclarations pour le capteur d'humidité DHT11.
#include <DHT11.h>
#define Pilotage_DH11 5
// Codes d'erreurs de la fonction readDHT11() :
enum {
  DHT11_SUCCESS,          // 0 : Pas d'erreur.
  DHT11_TIMEOUT_ERROR,    // 1 : Temps d'attente dépassé.
  DHT11_CHECKSUM_ERROR};  // 2 : Données reçues erronées.

// Déclarations pour le LOGO.
#include <EEPROM.h>
#define XL 14  // Position du LOGO.
#define YL 24 // Position du LOGO.

// Constantes du programme.
#define Test_manuel_des_capteurs 16 // A2 pour faire afficher la valeur de l'intervalle.
#define Alt_Locale_negative 16  // A2 utilisée en entrée pour Altitude Locale négative.
#define Entree_Altitude 17  // A3 utilisée en entrée pour ouvrir la fonction Altitude.
#define Sentinelle 4  // D4 utilisée en sortie pilotera l'alarme de température.
#define LED_Jaune 8  // LED d'avertissements divers.
#define BUZZER  10      // Sortie dédiée au BUZZER.
#define LED_Arduino 13
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.

// Variables du programme.
boolean Mode_securite_meteo;
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;
boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif;   // Sens de rotation.
boolean MDF_la_Valeur;
boolean Ligne_de_REF; // Signale qu'il faut afficher les pointillés de référence.
boolean Altitude_negative;
boolean Altimetre;
boolean Alarme_Sonore;
volatile boolean Rotation;  // Indique une activation du codeur rotatif.
byte OCTET, Ligne, Colonne, Mode_Ecran, VAL_centrale, Ordre, Position_INDEX;
byte Ancienne_Position_Curseur, Position_curseur;
byte PTR; // Pour le LOGO et le pointeur en RAM d'échantillons.
byte Pression; // Variable pour le capteur de pression.
// Pression sera égal à P - 850 pour contenir dans un byte.
byte Temperature; // Variable pour le capteur de pression.
byte Temp_Humid, Humidite;  // Variables pour le capteur d'humidité.
byte Heures, Minutes, Secondes;
byte VAL_saisie; // Gère le texte d'invite lors de la saisie d'une valeur.
byte Parametre; // Paramètre affiché sur le graphe.
byte T_MINI_autorisee; // Température en dessous de laquelle on déclenche l'alarme.
byte Courbe; // Trace qui sera validé en mode Exploitation.
long Chronometre, VALEUR, PAS;
unsigned int PAQUET;
long Correction_Altitude_barometrique;
unsigned long Ancienne_valeur_Temps_Ecoule_depuis_RESET;
unsigned long Delai_entre_Enregistrements; // Durée en seconde.
float Altitude, Reference_zero;
byte MEMOIRE_PRESSION[120];
byte MEMOIRE_HYGROMETRIE[120];
byte MEMOIRE_TEMPERATURE[120];

void setup() {
  Mode_securite_meteo = EEPROM.read(659);
  pinMode(BUZZER, INPUT); pinMode(Codeur_A, INPUT); pinMode(Codeur_B, INPUT);
  digitalWrite(BP_central, HIGH); // Valide R PUL-UP sur A1.
  digitalWrite(Test_manuel_des_capteurs, HIGH); // Valide R PUL-UP sur A2.  
  digitalWrite(Entree_Altitude, HIGH); // Valide R PUL-UP sur A3.  
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  // Broche de l'interruption 1 (E/S D3) :
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  pinMode(LED_Arduino, OUTPUT); digitalWrite(LED_Arduino, LOW);
  pinMode(LED_Jaune, OUTPUT); digitalWrite(LED_Jaune, LOW);
  pinMode(Sentinelle, OUTPUT); digitalWrite(Sentinelle, HIGH);
  display.initialize(); delay(300); display.clear(); display.setTextSize(1); // Taille la plus petite.
  if (Mode_securite_meteo) {Mode_Ecran = 2; // Mode VEILLE sur RESET.
                            Recharge_delai_depuis_EEPROM();}
  else {Mode_Ecran = 1; // Mode EXPLOITATION sur RESET.
        display.setCursor(12,4); Aff_TEXTE_EEPROM(407, 18); CADRE(); // Affiche la version.
        display.setTextSize(2); // Taille double.
        display.setCursor(65,21); Aff_TEXTE_EEPROM(120, 4);
        display.setCursor(57,37); Aff_TEXTE_EEPROM(124, 5);
        //================================= Tracer le LOGO. ===============================
        PTR = 0; // Pointer le premier octet dans l'EEPROM.
        for(Ligne = 0; Ligne < 30; Ligne++)
           {for(Colonne = 0; Colonne < 4; Colonne++) 
               {OCTET = EEPROM.read(PTR);
                for(byte I=0; I < 8; I++)
                    {display.drawPixel((Colonne*8)+XL+I, Ligne+YL, (OCTET & B10000000));
                     OCTET=OCTET<<1;}; PTR++; } }; 
        //=================================================================================
        display.setTextSize(1); display.setCursor(74,53); Aff_TEXTE_EEPROM(139, 5); display.update();
        Delai_entre_Enregistrements = 5040; // Graduations pour une semaine.
        Heures = 1; Minutes = 24; Secondes = 0; // Valeurs pour une semaine.
        ATTENDRE_un_BP();}
  display.setTextSize(1); Chronometre = 0; Altimetre = false; Alarme_Sonore = false;
  Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis();
  Ancienne_Position_Curseur = 0; // Ancienne position INDEX supposée nulle.
  Position_curseur = 40; // Valeur quelconque hors cadre jaune.
  Parametre = 0; Courbe=0; 
  // Dernière consigne de Température d'alarme récupérée en EEPROM :
  T_MINI_autorisee = EEPROM.read(660); Alarme_Sonore = EEPROM.read(658);
  Tester_les_capteurs(); Effacer_tous_les_echantillons(); 
  Ligne_de_REF = false;
  VAL_centrale = 119; // Position maximale pointée pour enregistrer.
//@@@@@@@@@@@ Ci-dessous code ajouté pour afficher la place disponible. @@@@@@@@@@@
//  display.clear(); display.setCursor(30,30); display.print(SRAM_LIBRE());
//  display.update(); ATTENDRE_un_BP();
//@@@@@@@@@@@@**********************************************************@@@@@@@@@@@
  Ecran_ENREGISTREUR();}
  
//@@@@@@@@@@@@@@ Ci-dessous fonction pour afficher la place disponible @@@@@@@@@@@@@@
//  int SRAM_LIBRE() { // Fonction qui retourne la taille de SRAM disponible.
//  extern int __heap_start, *__brkval; // Déclaration des deux pointeurs dédiés.
//  byte BIDON; // Dernière variable allouée, donc occupe le "haut" du TAS.
//  if (__brkval == 0) {return (int) &BIDON -(int) &__heap_start;}
//  else {return (int) &BIDON -(int) __brkval;} }
//@@@@@@@@@@@@@@*******************************************************@@@@@@@@@@@@@@

//================================= Fonction MÉTÉO ==================================
// Chronometre : Compteur dans la boucle pour faire pulser la LED jaune.
// Ordre : Indique la position horizontale sur la grille de l'INDEX. 0 à 9.
// Ancienne_Position_Curseur : dernière position du curseur vertical à effacer.
// VAL_centrale : Pointeur du prochain échantillon qui sera enregistré.

void loop() {
  if (millis() > 3900000000) Comptage_maximum(); // Message d'alerte à 45,14 jours.
  //==================== Effectuer un échantillonnage ====================
  if (millis() > Ancienne_valeur_Temps_Ecoule_depuis_RESET)  {
      Ancienne_valeur_Temps_Ecoule_depuis_RESET = // réarme immédiatement le délai.
         (unsigned long) (millis() + (Delai_entre_Enregistrements * 1000));
      if (Mode_Ecran != 0) Effectue_un_enregistrement();// Suivi d'un affichage de l'écran graphique.
      if (Mode_Ecran == 1) Affiche_P_H_T();} // Affiche les valeurs numériques.
  //======================== Traite le B.P. central ======================
  Traite_BP_Central();
  //========================= Traite la rotation =========================
  INT_active = true; // Armer un nouveau comptage.
  if (Rotation) { Rotation = false; 
      if (Mode_Ecran == 0) { // Affichage indépendant des paramètres.
          if (Sens_positif) {Ordre++; if (Ordre == 40) Ordre = 0;}
          else {if (Ordre > 0) Ordre--; else Ordre = 39;}
          Affiche_INDEX_ENREGISTREMENT();}
      if (Mode_Ecran == 1) { // Mode exploitation.
          if (Sens_positif) {Ecran_des_donnees_enregistreur(); Affiche_consigne_temperature();}
          else Affiche_les_valeurs_des_graduations();
          Ecran_ENREGISTREUR();}
      if (Mode_Ecran == 2) {  // Mode veille.
          if (Sens_positif) Tester_les_capteurs(); else Afficher_les_extremums();
          ATTENDRE_un_BP(); Ecran_ENREGISTREUR();} }
  //============== Traite A2 l'inverseur basculé à gauche ================
  // Si mode Historique : Valide ou efface les quatre lignes pointillées.
  // Si mode EXPLOITATION : Remise à zéro du graphe.
  // Si mode VEILLE écran noir : Change le mode SECURITE.
  if (analogRead(Test_manuel_des_capteurs) < 512) {// Si A2 est activée :
      digitalWrite(LED_Arduino, HIGH); // Allume la LED.
      while (analogRead(Test_manuel_des_capteurs) < 512); // Attendre le relâché.
      digitalWrite(LED_Arduino, LOW);
      switch (Mode_Ecran) {
        case 0 : {Ligne_de_REF = !Ligne_de_REF; break;}
        case 1 : {Remise_a_zero_du_graphe(); break;}
        case 2 : Changer_le_mode_SECURITE_meteo(); }
      Ecran_ENREGISTREUR(); }     
  //============ Si A3 est activée traite la fonction ALTITUDE ===========
  if (analogRead(Entree_Altitude) < 512) Confirmer_ALTITUDE();
  //=============== Traiter la requête de l'un des deux B.P. =============
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
      Clic_sur_le_BP = false; // Action sur le B.P. prise en compte. 
      if (Clic_long) {// Traite le B.P. long.
          if (Augmenter_le_choix) Saisir_Intervalle(); else Saisir_T_alarme(); }
     else {// Traite clic court :
           if (Mode_Ecran == 0) // Affichage indépendant des paramètres.
               {if (Augmenter_le_choix) {Parametre++; if (Parametre == 3) Parametre = 0;}
                else {if (Parametre > 0) Parametre--; else Parametre = 2;}
               Ecran_ENREGISTREUR();}
           if (Mode_Ecran == 1) // Mode exploitation.
               {if (Augmenter_le_choix) Sauvegarde_les_Echantillons_en_EEPROM();
                else Recharger_les_donnees_depuis_EEPROM();}
           if (Mode_Ecran == 2) // Mode veille.
               {if (Augmenter_le_choix)
                {Ecran_des_donnees_enregistreur(); Affiche_consigne_temperature(); Ecran_ENREGISTREUR();} 
                else Visualiser_temporairement();}}}
  // ======= Traiter la LED jaune du mode VEILLE et la Sentinelle ========
  Chronometre++; // Faire clignoter si VEILLE ou trois paramètres.
  if (Chronometre == 6000) {Chronometre = 0; // Réarme le chronomètre.
      if (Mode_Ecran == 1) {Courbe++; if (Courbe == 3) Courbe = 0; Ecran_ENREGISTREUR();} 
      if (Mode_Ecran != 0) {digitalWrite(LED_Jaune,HIGH); delay(100); digitalWrite(LED_Jaune,LOW);}
      Mesurer_les_trois_parametres(); // Ne fait que mesurer Humidite, Pression et Temperature. 
      if (Temperature-100 < T_MINI_autorisee)
         {digitalWrite(Sentinelle, LOW); if (Alarme_Sonore) BIP();}
         else digitalWrite(Sentinelle, HIGH);
      if (Mode_Ecran == 1) Affiche_P_H_T(); } }
//============================= Fin de la boucle de base ====================================

void Traite_BP_Central() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
      digitalWrite(LED_Arduino, HIGH); // Allume la LED.
      while (analogRead(BP_central) < 512); // Attendre le relâché.
      digitalWrite(LED_Arduino, LOW);
      if (Altimetre) {if (Mode_Ecran == 0) Mode_Ecran = 1; else Mode_Ecran = 0;
                      if (Mode_Ecran == 1) {Affiche_ecran_altimetre(); Mesure_Reference_zero();} }
      else {Mode_Ecran++; if (Mode_Ecran == 3) Mode_Ecran = 0;
            if (Mode_Ecran == 1) Chronometre == 5999; // Pour affichage des valeurs immédiat.
            Ecran_ENREGISTREUR();} } }

void Aff_pourcent() {display.print("%");}
void Aff_degre_celcius() {display.print(char(247)); display.print("C");}

void Afficher_Entete_pour_trois_valeurs() {
  Affiche_cadre_du_haut(32); Aff_TEXTE_EEPROM(398, 3); display.setCursor(76,4); Aff_pourcent();
  display.setCursor(110,4); display.print(char(247)); display.print("C");
  display.drawLine(54, 1, 54, 14,WHITE); // Séparation de gauche.
  display.drawLine(87, 1, 87, 14,WHITE);} // Séparation de droite.

void Afficher_les_extremums(){
  Afficher_Entete_pour_trois_valeurs(); // Mise en place du cadre jaune et des unités.
  display.setCursor(38,25); Aff_TEXTE_EEPROM(205, 3); // Affiche "Min".
  display.setCursor(38,45); Aff_TEXTE_EEPROM(264, 3); // Affiche "MAX".
  //------------------------ Afficher les Minima ------------------------
  // Recherche la Pression la plus petite :
  VAL_saisie = MEMOIRE_PRESSION[0];
  for (PTR = 1; PTR < 120; PTR++)
       if(MEMOIRE_PRESSION[PTR] < VAL_saisie) VAL_saisie = MEMOIRE_PRESSION[PTR];
  Effacer_zone(7, 25, 4); display.setCursor(7,25); if ((VAL_saisie + 850) < 1000) Affiche_1_espace(); 
  display.print(VAL_saisie + 850);
  // Recherche l'Humidité la plus petite :
  VAL_saisie = MEMOIRE_HYGROMETRIE[0];
  for (PTR = 1; PTR < 120; PTR++)
       if(MEMOIRE_HYGROMETRIE[PTR] < VAL_saisie) VAL_saisie = MEMOIRE_HYGROMETRIE[PTR];
  Effacer_zone(62, 25, 2); display.setCursor(62,25); display.print(VAL_saisie);
  // Recherche la Température la plus petite :
  VAL_saisie = MEMOIRE_TEMPERATURE[0];
  for (PTR = 1; PTR < 120; PTR++)
       if(MEMOIRE_TEMPERATURE[PTR] < VAL_saisie) VAL_saisie = MEMOIRE_TEMPERATURE[PTR];
  Effacer_zone(91, 25, 3); display.setCursor(92,25);
      if (abs(VAL_saisie - 100) < 10) Affiche_1_espace();
      if (VAL_saisie - 100 >= 0) display.print("+"); display.print(VAL_saisie - 100);    
  //------------------------ Afficher les Maxima ------------------------
  // Recherche la Pression la plus grande :
  VAL_saisie = MEMOIRE_PRESSION[0];
  for (PTR = 1; PTR < 120; PTR++)
       if(MEMOIRE_PRESSION[PTR] > VAL_saisie) VAL_saisie = MEMOIRE_PRESSION[PTR];
  Effacer_zone(7, 45, 4); display.setCursor(7,45); if ((VAL_saisie + 850) < 1000) Affiche_1_espace(); 
  display.print(VAL_saisie + 850);
  // Recherche l'Humidité la plus grande :
  VAL_saisie = MEMOIRE_HYGROMETRIE[0];
  for (PTR = 1; PTR < 120; PTR++)
       if(MEMOIRE_HYGROMETRIE[PTR] > VAL_saisie) VAL_saisie = MEMOIRE_HYGROMETRIE[PTR];
  Effacer_zone(62, 45, 2); display.setCursor(62,45); display.print(VAL_saisie);
  // Recherche la Température la plus grande :
  VAL_saisie = MEMOIRE_TEMPERATURE[0];
  for (PTR = 1; PTR < 120; PTR++)
       if(MEMOIRE_TEMPERATURE[PTR] > VAL_saisie) VAL_saisie = MEMOIRE_TEMPERATURE[PTR];
  Effacer_zone(91, 45, 3); display.setCursor(92,45);
      if (abs(VAL_saisie - 100) < 10) Affiche_1_espace();
      if (VAL_saisie - 100 >= 0) display.print("+"); display.print(VAL_saisie - 100);    
  Mode_Ecran = 1; Affiche_P_H_T(); // Effectue aussi display.update.
  Mode_Ecran = 2; } // Rétablir le mode VEILLE.
  
void Changer_le_mode_SECURITE_meteo() {
  Mode_securite_meteo = !Mode_securite_meteo;
  if (Mode_securite_meteo) EEPROM.write(659,1); else EEPROM.write(659,0);
  display.clear(); display.setCursor(10,25); Aff_TEXTE_EEPROM(448,14);
  display.setCursor(50,40);
  if (Mode_securite_meteo) Aff_TEXTE_EEPROM(463,3); else Aff_TEXTE_EEPROM(252,3); 
  display.update(); ATTENDRE_un_BP(); Ecran_ENREGISTREUR();}

void Remise_a_zero_du_graphe() {
  BIP(); display.clear(); display.setCursor(2,32); Aff_TEXTE_EEPROM(426, 21);
  display.update(); ATTENDRE_un_BP(); if (Augmenter_le_choix) Effacer_tous_les_echantillons();
  Ecran_ENREGISTREUR();}

void Effacer_tous_les_echantillons() {
  for(byte Echantillon = 0; Echantillon < 120; Echantillon++) {
      MEMOIRE_PRESSION[Echantillon] = 90; MEMOIRE_HYGROMETRIE[Echantillon] = 0;
      MEMOIRE_TEMPERATURE[Echantillon] = 100;} }

void Affiche_les_valeurs_des_graduations() {
  // Bien qu'il ne s'agisse que d'afficher des nombres constants, cette procédure fonctionne
  // par du calcul des valeurs car ainsi on n'affiche pas de chaînes de caractères qui
  // consomment trop de mémoire dynamique.
  byte Y;
  Efface_interieur_cadre_jaune(); display.setCursor(7,4); Aff_TEXTE_EEPROM(505, 17);
  
  switch (Delai_entre_Enregistrements) {
        case 20160 : {Aff_TEXTE_EEPROM(524,2); break;} // Delai pour 28 jours.
        case 5040 : {Aff_TEXTE_EEPROM(522,2); break;} // Delai pour une semaine.
        case 720 : {Aff_TEXTE_EEPROM(526,2); break;} // Delai pour 1J jours.
        default : display.drawLine(4, 4, 104, 10,WHITE); }

  for (byte Valeur = 0; Valeur < 6; Valeur++) {Y = 17 + (Valeur * 8);
       Effacer_zone(10, Y, 18); display.setCursor(17,Y);
       if (1040 - (Valeur * 20) < 1000) Affiche_1_espace();
       display.print(1040 - (Valeur * 20));}      
  for (byte Valeur = 0; Valeur < 6; Valeur++) {Y = 17 + (Valeur * 8);
       display.setCursor(60,Y);
       if (100 - (Valeur * 20) < 100) Affiche_1_espace();
       if (100 - (Valeur * 20) < 10) Affiche_1_espace();
       display.print(100 - (Valeur * 20));}   
   for (byte Valeur = 0; Valeur < 6; Valeur++) {Y = 17 + (Valeur * 8);
       display.setCursor(97,Y);
       if (Valeur == 5) display.setCursor(91,57);
       display.print(40 - (Valeur * 10));}
  display.update(); ATTENDRE_un_BP();}

//================================= Fonction ALTIMETRE ======================================
void Saisie_Correction_Altitude() {// VALEUR et Correction_Altitude_barometrique sont de type long.
  float Altitude;
  // Dans cette procédure on indique l'altitude réelle portée sur la carte du randonneur.
  VAL_saisie = 5; Saisie_VALEUR(); if (Altitude_negative) VALEUR = 0 - VALEUR;
  Altitude = Capte_Altitude_sans_correction();
  Correction_Altitude_barometrique = long(VALEUR - long(Altitude));
  Affiche_ecran_altimetre(); Clic_sur_le_BP = false;}

void Mesure_Reference_zero() {
  Reference_zero = Mesure_Altitude(); Effacer_zone(76, 4, 8);
  display.setCursor(76,4); display.print(Reference_zero,1 ); Aff_ESP_m(); display.update();}
  
void Aff_ESP_m() {Aff_TEXTE_EEPROM(401, 2);}

float Capte_Altitude_sans_correction() {
  float Altitude = 0.0;
  for(byte Nb_Ech = 0; Nb_Ech < 50; Nb_Ech++) {
      Tester_le_BP_A0(); // Pour prise en compte durant les mesures.
      Altitude = Altitude + Capteur_BMP085.readAltitude();}
  return (Altitude / 50.0);}
  
float Mesure_Altitude() {
  float Altitude = Capte_Altitude_sans_correction();
  return Altitude + Correction_Altitude_barometrique;}
  
void Affiche_ecran_altimetre() {
  Clic_long = false; display.clear();
  Affiche_cadre_du_haut(10);
  Aff_TEXTE_EEPROM(355, 10); display.setCursor(10,22); Aff_TEXTE_EEPROM(334, 10);
  display.setCursor(10,36); Aff_TEXTE_EEPROM(334, 10); CADRE();
  display.setCursor(10,50); Aff_TEXTE_EEPROM(344, 10); display.update();}
  
void Confirmer_ALTITUDE() {
  BIP(); display.clear(); display.setCursor(2,32); Aff_TEXTE_EEPROM(488, 16);
  display.update(); ATTENDRE_un_BP(); if (Augmenter_le_choix) Procedure_ALTITUDE();
  Ecran_ENREGISTREUR();}
  
void Procedure_ALTITUDE(){
  Correction_Altitude_barometrique = 0; Altimetre = true;
  // Ci-dessous Affiche_ecran_altimetre() force à false la variable Clic_long.
  Affiche_ecran_altimetre(); Mesure_Reference_zero();
  //---------------------------- Tant que NON sortie --------------------------------
  while (!Clic_long) {
    if (Mode_Ecran == 1) {
        Effacer_zone(76, 22, 8); display.setCursor(76,22);
        Altitude = Mesure_Altitude();
        display.print(Altitude,1 ); Aff_ESP_m();
        Effacer_zone(76, 36, 8); display.setCursor(76,36);
        display.print(Altitude/0.3048,0 ); Aff_TEXTE_EEPROM(404, 3);
        Effacer_zone(76, 50, 8); display.setCursor(76,50);
        display.print(Altitude - Reference_zero,0 ); Aff_ESP_m();}
    else {display.clear(); delay(3000);}
    display.update();  
    digitalWrite(LED_Jaune, HIGH); delay(100); digitalWrite(LED_Jaune, LOW);
    //========================= Traite la rotation =========================
    INT_active = true; // Armer un nouveau comptage.
    if (Rotation) { Rotation = false; BIP();}
    //======================== Traite le B.P. central ======================
    Traite_BP_Central();
    //=============== Traiter la requête de l'un des deux B.P. =============
    Tester_le_BP_A0();
    if (Clic_sur_le_BP) {Clic_sur_le_BP = false; // Action sur le B.P. prise en compte.
       if (Clic_long) {BIP(); display.clear(); display.setCursor(2,32); Aff_TEXTE_EEPROM(488, 5);
           Aff_TEXTE_EEPROM(457, 5); Aff_TEXTE_EEPROM(445, 2); display.update(); ATTENDRE_un_BP();
           Affiche_ecran_altimetre(); // Attention : Force Clic_long à false. 
           Clic_long = Augmenter_le_choix;  // Sortir de l'Altimètre si FC+.
           display.setCursor(76,4); display.print(Reference_zero,1);} 
       else {if (Augmenter_le_choix) Saisie_Correction_Altitude(); Mesure_Reference_zero();} } }
  //---------------------------------------------------------------------------------  
  Clic_sur_le_BP = false; Mode_Ecran = 1; Altimetre = false;}
//============================ Fin de la fonction ALTIMETRE =================================
      
void Visualiser_temporairement() {
  Mode_Ecran = 1; Ecran_ENREGISTREUR(); Affiche_P_H_T();
  ATTENDRE_un_BP(); Mode_Ecran = 2; Ecran_ENREGISTREUR();} 
      
void Sauvegarde_les_Echantillons_en_EEPROM() {
  BIP(); display.clear(); display.setCursor(32,18); Aff_TEXTE_EEPROM(311, 10); display.setCursor(28,29);
  Aff_TEXTE_EEPROM(322, 11); display.update(); ATTENDRE_un_BP();
  if (Augmenter_le_choix) {
      for(byte Echantillon = 0; Echantillon < 120; Echantillon++) {
         EEPROM.write(Echantillon + 664, MEMOIRE_PRESSION[Echantillon]);
         EEPROM.write(Echantillon + 784, MEMOIRE_HYGROMETRIE[Echantillon]);
         EEPROM.write(Echantillon + 904, MEMOIRE_TEMPERATURE[Echantillon]);} }     
  Ecran_ENREGISTREUR();}
  
void Recharger_les_donnees_depuis_EEPROM() {
  BIP(); display.clear(); display.setCursor(26,18); Aff_TEXTE_EEPROM(297, 13); display.setCursor(14,29);
  Aff_TEXTE_EEPROM(149, 17); display.update(); ATTENDRE_un_BP();
  if (Augmenter_le_choix) {Rappel_echantillons_EEPROM(); VAL_centrale = 119;}
  Ecran_ENREGISTREUR();}
      
void Saisir_T_alarme() {
  BIP(); display.clear(); display.setCursor(2,32); Aff_TEXTE_EEPROM(467, 20);
  display.update(); ATTENDRE_un_BP();
  if (Augmenter_le_choix) {
    VAL_saisie = 4; Saisie_VALEUR(); T_MINI_autorisee = VALEUR;
    if (T_MINI_autorisee > 45) T_MINI_autorisee = 45; Mode_Ecran = 1;
    EEPROM.write(660, T_MINI_autorisee);} // Sauvegarde la valeur de consigne en EEPROM.
  display.clear(); display.setCursor(2,32); Aff_TEXTE_EEPROM(529, 15);
  display.update(); ATTENDRE_un_BP(); Alarme_Sonore = Augmenter_le_choix;
  EEPROM.write(658, Alarme_Sonore);
  Affiche_consigne_temperature(); Ecran_ENREGISTREUR();}
  
void Affiche_consigne_temperature() {
  display.clear(); display.setCursor(28,25); Affiche_Degre_C_Alarme(); Aff_TEXTE_EEPROM(227, 3);
  display.print(T_MINI_autorisee); display.setCursor(20,45); Aff_TEXTE_EEPROM(529, 14);
  if (Alarme_Sonore) Aff_TEXTE_EEPROM(463, 3); else Aff_TEXTE_EEPROM(252, 3);
  display.update(); ATTENDRE_un_BP();}

void Saisie_VALEUR() {
  VALEUR = 0; PAS = 1; MDF_la_Valeur = true; Altitude_negative = false;
  Ecran_Saisie_VALEUR(); A_set = true; boolean B_set = true; INT_active = false;
  Sens_positif = true; Clic_long = false; Clic_sur_le_BP = false;
  while (!Clic_long) {
  //================================ Tant que non SORTIE ===========================
  //============== Traite A6 pour Altitude locale négative ===============
  if (analogRead(Alt_Locale_negative) < 512) {// Si A6 est activée :
      digitalWrite(LED_Arduino, HIGH); // Allume la LED.
      while (analogRead(Alt_Locale_negative) < 512); // Attendre le relâché.
      digitalWrite(LED_Arduino, LOW);
      if (VAL_saisie == 5) {Altitude_negative = !Altitude_negative;
          Effacer_zone(14, 56, 18);
          if (Altitude_negative)
             {display.setCursor(14,56); Aff_TEXTE_EEPROM(380,18);}
          display.update();}
      else BIP();}
  //======================== Traite le B.P. central ======================
   if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
      digitalWrite(LED_Arduino, HIGH); // Allume la LED.
      while (analogRead(BP_central) < 512); // Attendre le relâché.
      digitalWrite(LED_Arduino, LOW); MDF_la_Valeur = !MDF_la_Valeur; Affiche_le_MODE(); }
  //========================= Traite la rotation =========================
  INT_active = true; // Armer un nouveau comptage.
  if (Rotation) {Rotation=false;
      if (MDF_la_Valeur) {MAJ_VALEUR_avec_limites(); Affiche_VALEUR();}
      else Modifie_le_PAS(); } 
  //=============== Traiter la requête de l'un des deux B.P. =============
  Tester_le_BP_A0();
  if ((Clic_sur_le_BP) && (!Clic_long)) {
     Clic_sur_le_BP = false; // Action sur le B.P. prise en compte.
     // Traitement de l'option.
     if (Augmenter_le_choix) {// Traite le B.P. FC+ court ----------------
        VALEUR = VALEUR * 10;
        if (VALEUR > 999) {VALEUR = 999; BIP();}
        Affiche_VALEUR();}
     else {// ----------------- Traite le B.P. FC- court -----------------
           VALEUR = 0; PAS = 1;
           Affiche_le_PAS();  MDF_la_Valeur = true; Affiche_le_MODE();
           Affiche_VALEUR();  } } }
  Clic_long = false; }
  
void Ecran_des_donnees_enregistreur() {
  display.clear(); Affiche_cadre_du_haut(26); Aff_TEXTE_EEPROM(231, 11);
  display.print((float) Delai_entre_Enregistrements * 120 / 86400,1);
  display.setCursor(26,19); Aff_TEXTE_EEPROM(192, 9); display.print(Heures);
  display.setCursor(26,30); Aff_TEXTE_EEPROM(205, 10); display.print(Minutes);
  display.setCursor(26,41); Aff_TEXTE_EEPROM(219, 11); display.print(Secondes);
  display.setCursor(19,54); Aff_TEXTE_EEPROM(149, 15); display.drawLine(16, 51, 112, 63,WHITE);
  display.update(); ATTENDRE_un_BP(); Ecran_ENREGISTREUR();}
  
void Affiche_Degre_C_Alarme()
  {display.print(char(247)); Aff_TEXTE_EEPROM(288, 8);}
  
void AFF_59() {Aff_TEXTE_EEPROM(372,2);}
  
void Ecran_Saisie_VALEUR() {
  display.clear(); Affiche_cadre_du_haut(8); Aff_TEXTE_EEPROM(279,8);
  switch(VAL_saisie) {
    case 1 : {Aff_TEXTE_EEPROM(189,9); display.setCursor(13,45); display.print("8"); break;}
             // Ci-dessus : "es Heures".
    case 2 : {Aff_TEXTE_EEPROM(202,10); display.setCursor(11,45); AFF_59(); break;}
             // Ci-dessus : "es Minutes".
    case 3 : {Aff_TEXTE_EEPROM(216,11); display.setCursor(11,45); AFF_59(); break;}
             // Ci-dessus : "es Secondes".
    case 4 : {Aff_TEXTE_EEPROM(341,2); Affiche_Degre_C_Alarme(); display.setCursor(11,45);
              Aff_TEXTE_EEPROM(259,2); break;}
    case 5 : {display.print("'"); Aff_TEXTE_EEPROM(334,8);
              display.setCursor(7,45); Aff_TEXTE_EEPROM(373,3);} }       
  display.setCursor(7,35); Aff_TEXTE_EEPROM(145,3);
  Affiche_le_MODE(); Affiche_le_PAS(); Affiche_VALEUR();
  display.update();}
  
void MAJ_VALEUR_avec_limites() {
  if (Sens_positif) {VALEUR = VALEUR + PAS;
       if (VALEUR > 999) {VALEUR = 999; BIP();} }
  else {VALEUR = VALEUR - PAS;
       if (VALEUR < 0) {VALEUR = 0; BIP();} } }

void Saisir_la_duree_entre_enregistrements() {
  VAL_saisie = 1; Saisie_VALEUR(); Heures = VALEUR; if (Heures > 8) Heures = 8;
  VAL_saisie = 2; Saisie_VALEUR(); Minutes = VALEUR; if (Minutes > 59) Minutes = 59;
  VAL_saisie = 3; Saisie_VALEUR(); Secondes = VALEUR; if (Secondes > 59) Secondes = 59;
  Calcule_DELAI_entre_Echantillons(); Ecran_des_donnees_enregistreur(); Mode_Ecran = 1;}
  
void Affiche_le_PAS() {Effacer_zone(92, 44, 2);
  display.setCursor(56,44); Aff_TEXTE_EEPROM(366, 6); display.print(PAS); display.update();}

void Modifie_le_PAS(){if (PAS == 1) PAS = 10;  else PAS = 1; Affiche_le_PAS();}
  
void Affiche_curseur(byte X, byte Y) {
     Effacer_zone(X, Position_curseur, 1); display.setCursor(X,Y);
     display.print(char(17)); display.update(); Position_curseur = Y; }
  
void Affiche_VALEUR() {
  display.setCursor(39,27); Aff_TEXTE_EEPROM(130,8); Affiche_VALEUR_avec_formatage(93, 27, VALEUR);}
  
void Affiche_VALEUR_avec_formatage(byte X, byte Y, long VALEUR) {
  // Formate pour 3 caractères et efface le champ complet avant affichage.
  Effacer_zone(X, Y, 3); display.setCursor(X,Y);
  if (VALEUR < 100) Affiche_1_espace(); if (VALEUR < 10) Affiche_1_espace();
  display.print(VALEUR); display.update(); }
  
void Affiche_le_MODE() {// Affiche l'index du MODE de modification.
  if (MDF_la_Valeur) Affiche_curseur(119,27);
  else  Affiche_curseur(119,43);}

void Saisir_Intervalle() {
  BIP(); display.clear(); display.setCursor(38,18); Aff_TEXTE_EEPROM(167, 8);
  display.setCursor(26,28); Aff_TEXTE_EEPROM(176, 12); display.update(); ATTENDRE_un_BP();
  if (Augmenter_le_choix)
     {Saisir_la_duree_entre_enregistrements(); EEPROM.write(661,Heures); EEPROM.write(662,Minutes);
      EEPROM.write(663,Secondes);}
  Ecran_ENREGISTREUR();}

void Effectue_un_enregistrement() {
  if (VAL_centrale == 119) // Decale_tous_les_echantillons.
      for (byte I=1; I<120; I++) {
        MEMOIRE_PRESSION[I-1] = MEMOIRE_PRESSION[I];
        MEMOIRE_HYGROMETRIE[I-1] = MEMOIRE_HYGROMETRIE[I];
        MEMOIRE_TEMPERATURE[I-1] = MEMOIRE_TEMPERATURE[I];}
  else VAL_centrale++; // Passe à l'échantillon voisin.
  Mesurer_les_trois_parametres();
  MEMOIRE_PRESSION[VAL_centrale] = Pression;
  MEMOIRE_HYGROMETRIE[VAL_centrale] = Humidite;
  MEMOIRE_TEMPERATURE[VAL_centrale] = Temperature;
  Ecran_ENREGISTREUR();} 

void Mesurer_les_trois_parametres() {
  readDHT11(Pilotage_DH11, &Temp_Humid, &Humidite);
  Pression = (Capteur_BMP085.readPressure()/100) - 850;
  Temperature = char (Capteur_BMP085.readTemperature()) + 100;  // Arrondir au °C le plus proche :
  if (abs(Capteur_BMP085.readTemperature())- abs(int(Capteur_BMP085.readTemperature())) > 0.5)
      if (Capteur_BMP085.readTemperature()>0) Temperature++; else Temperature--;}
       
// Mode_Ecran = 0 : CONSULTATION : Écran épuré avec B.P. court pour le choix du paramètre affiché.
//            = 1 : Écran avec les trois paramètres affichés. Valeurs numériques actuelles
//            = 2 : Écran. (Mode veille)
//  Parametre = 0 : Affiche la pression atmosphérique.
//            = 1 : Affiche l'hygrométrie.
//            = 2 : Affiche la température

void Affiche_tendances() {
  Mesurer_les_trois_parametres(); VAL_saisie = 0; 
  switch(Parametre) {
    case 0 : {// Détermine la tendance pour la Pression.
              if (Pression < MEMOIRE_PRESSION[119]) VAL_saisie = 2;
              if (Pression > MEMOIRE_PRESSION[119]) VAL_saisie = 1; break;}
    case 1 : {// Détermine la tendance pour l'Hygrométrie.
              if (Humidite < MEMOIRE_HYGROMETRIE[119]) VAL_saisie = 2;
              if (Humidite > MEMOIRE_HYGROMETRIE[119]) VAL_saisie = 1; break;}             
    case 2 : {// Détermine la tendance pour la Température.
              if (Temperature < MEMOIRE_TEMPERATURE[119]) VAL_saisie = 2;
              if (Temperature > MEMOIRE_TEMPERATURE[119]) VAL_saisie = 1;} }
  Pointe_bas_gauche();
  switch(VAL_saisie) {
    case 0 : {display.print(char(23)); break;} // Flèche Double
    case 1 : {display.print(char(24)); break;} // Flèche Haut
    case 2 :  display.print(char(25)); } } // Flèche Bas  

void Pointe_bas_gauche() { display.setCursor(5,53);
  if(Mode_Ecran == 0) Effacer_zone(5, 53, 1); else Effacer_zone(5, 53, 3); display.setCursor(5,53);}    

void Ecran_ENREGISTREUR(){
  display.clear();
  if (Mode_Ecran != 2) {// Si différent du mode Veille :
     if (Mode_Ecran == 0) {
         Affiche_E_croix(); Affiche_tendances();
         switch(Parametre) {
         case 0 : {display.drawRect(1, 1, 53, 14, WHITE); break;} 
         case 1 : {display.drawRect(55, 1, 32, 14, WHITE); break;}
         case 2 : display.drawRect(88, 1, 39, 14, WHITE); } }
     else display.drawRect(1, 1,126, 14, WHITE);
     Afficher_Entete_pour_trois_valeurs(); 
     // Ci-dessous trace le cadre bleu des données.
     display.drawRect(0, 18, 121, 46, WHITE); // Coordonnées et taille.
     //================ Trace les six niveaux à droite ===============
     for (byte I=18; I < 72; I=I+9)display.drawLine(123, I, 127, I, WHITE);
     //====== Trace les quatre lignes pointillées Horizontales =======
     if (Ligne_de_REF) {
         if (Mode_Ecran == 0) {
            for (byte I=27; I < 60; I=I+9) 
                 for (byte J=0; J < 120; J=J+6) display.drawLine(J+1, I, J+2, I, WHITE);}
         display.drawPixel(PTR, OCTET, WHITE);}
     //====== Trace les graduations journée / semaine / 28 jours =====    
     Graduations_JOUR_SEMAINE();
     //====================== Afficher la courbe =====================
     for (PTR = 0; PTR < 120; PTR++) Trace_un_triplet_de_mesures();
     if (Mode_Ecran == 0) Affiche_INDEX_ENREGISTREMENT();}
  if (Mode_Ecran == 1) {
      Pointe_bas_gauche();
      switch(Courbe) {
        case 0 : {Aff_TEXTE_EEPROM(398, 3); break;} // hPa
        case 1 : {Aff_pourcent(); break;} // %
        case 2 : Aff_degre_celcius(); } } // °C
  display.update();}
  
void Affiche_E_croix() {
  display.drawRect(121, 54, 7, 10, WHITE); display.setTextColor(BLACK,WHITE);
  display.setCursor(122,55); display.print("E"); display.setTextColor(WHITE);}
  
void Comptage_maximum() {
  Efface_interieur_cadre_jaune(); Affiche_cadre_du_haut(8);
  Aff_TEXTE_EEPROM(259, 19); display.update(); ATTENDRE_un_BP();}
  
void Affiche_INDEX_ENREGISTREMENT() { // Ancienne_Position_Curseur : Dernière position du curseur.
  Position_INDEX = Ordre * 3;
  if(!((Ancienne_Position_Curseur==0) || (Ancienne_Position_Curseur == 120))) // N'efface pas le CADRE.
     display.drawLine(Ancienne_Position_Curseur, 19, Ancienne_Position_Curseur, 62,BLACK);
     // Ci-dessus : Efface la ligne verticale.
  PTR = Ancienne_Position_Curseur;
  Trace_un_triplet_de_mesures(); // Retrace les trois points.
  display.drawLine(Position_INDEX, 19, Position_INDEX, 62,WHITE); // Trace la ligne V.
  // Efface les deux lignes du CURSEUR.
  display.drawLine(0, 16, 122, 16, BLACK); display.drawLine(0, 17, 122, 17, BLACK);
  // Trace le curseur.
  display.drawLine(Position_INDEX-2, 16, Position_INDEX+2, 16,WHITE);
  display.drawLine(Position_INDEX-1, 17, Position_INDEX+1, 17,WHITE);
  Affiche_tendances(); PTR = Position_INDEX; Affiche_un_triplet_de_valeurs();
  Ancienne_Position_Curseur = Position_INDEX; }
  
void Affiche_un_triplet_de_valeurs() {// Affiche le curseur et les deux valeurs
  //  numériques correspondant au centre de la graduation indexée par Ordre.
  Pression = MEMOIRE_PRESSION[PTR]; Humidite = MEMOIRE_HYGROMETRIE[PTR];
  Temperature = MEMOIRE_TEMPERATURE[PTR]; Affiche_P_H_T();}
  
void Affiche_P_H_T() {
  // Affiche la pression.
  if ((Parametre == 0) || (Mode_Ecran == 1))
     {Effacer_zone(7, 4, 4); display.setCursor(7,4); if ((Pression + 850) < 1000) Affiche_1_espace(); 
      display.print(Pression + 850);}
  // Affiche l'Humidité.
  if ((Parametre == 1) || (Mode_Ecran == 1))
     {Effacer_zone(62, 4, 2); display.setCursor(62,4); display.print(Humidite);}
  // Affiche la température.
  if ((Parametre == 2) || (Mode_Ecran == 1))
     {Effacer_zone(91, 4, 3); display.setCursor(92,4);
      if (abs(Temperature - 100) < 10) Affiche_1_espace();
      if (Temperature - 100 >= 0) display.print("+"); display.print(Temperature - 100);}
   display.update();}
  
void Trace_un_triplet_de_mesures() {
  OCTET = MEMOIRE_PRESSION[PTR]; if (OCTET < 90) OCTET = 90;
  if (OCTET > 190) OCTET = 190; OCTET = map(OCTET, 90, 190, 63, 18);
  if ((Parametre == 0 && Mode_Ecran == 0) || (Courbe == 0 && Mode_Ecran == 1))
       display.drawPixel(PTR, OCTET, WHITE); OCTET = MEMOIRE_HYGROMETRIE[PTR];
       OCTET = map(OCTET, 0, 100, 63, 18);
  if ((Parametre == 1 && Mode_Ecran == 0) || (Courbe == 1 && Mode_Ecran == 1))
       display.drawPixel(PTR, OCTET, WHITE); OCTET = MEMOIRE_TEMPERATURE[PTR];
       if (OCTET < 90) OCTET = 90; if (OCTET > 140) OCTET = 140; OCTET = map(OCTET, 90, 140, 63, 18);
  if ((Parametre == 2 && Mode_Ecran == 0) || (Courbe == 2 && Mode_Ecran == 1))     
       display.drawPixel(PTR, OCTET, WHITE);
  Graduations_JOUR_SEMAINE();}
  
void Rappel_echantillons_EEPROM() { // Restitue les échantillons sauvegardés en EEPROM.
  for(byte Echantillon = 0; Echantillon < 120; Echantillon++) {
      MEMOIRE_PRESSION[Echantillon] = EEPROM.read(Echantillon + 664);
      MEMOIRE_HYGROMETRIE[Echantillon] = EEPROM.read(Echantillon + 784);
      MEMOIRE_TEMPERATURE[Echantillon] = EEPROM.read(Echantillon + 904);} }

void Recharge_delai_depuis_EEPROM() {  
  Heures = EEPROM.read(661); Minutes = EEPROM.read(662); Secondes = EEPROM.read(663);
  Calcule_DELAI_entre_Echantillons();} 

//-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
void Delay_antirebonds() {for(int I=0; I<3; I++);} 

void Front_sur_A(){ // Interruption quand A change d'état avant B.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
  if (digitalRead(Codeur_A) != A_set ) {// Test pour savoir si l'état a changé.
    A_set = !A_set;
    if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();} } }

void Front_sur_B(){ // Interruption quand B change d'état avant A.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
  if( digitalRead(Codeur_B) != B_set ) {// Test pour savoir si l'état a changé.
    B_set = !B_set;
    // Décrémenter le compteur si B avant A.
    if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();} } }
//--------------------------------------------------------------------------------------------------

void Calcule_DELAI_entre_Echantillons() {
  Delai_entre_Enregistrements = (unsigned long) (3600 * Heures) + (60 * Minutes) + Secondes;
  if (Delai_entre_Enregistrements == 0) {Delai_entre_Enregistrements = 1; Secondes = 1;}
  Ancienne_valeur_Temps_Ecoule_depuis_RESET = 0;} // Provoquer un affichage immédiat.
  
void Tester_les_capteurs() {
  // Tant qu'aucun capteur d'humidité DHT11 n'est détecté boucle en ERREUR.
  while (readDHT11(Pilotage_DH11, &Temp_Humid, &Humidite) != DHT11_SUCCESS)
         Message_erreur_de_non_presence_capteurs();
  // Tant qu'aucun capteur de pression BMP085 n'est détecté boucle en ERREUR.
  while (!Capteur_BMP085.begin()) Message_erreur_de_non_presence_capteurs();
  Aff_non_presence_capteurs();
  for (byte X=74; X < 92; X++) display.drawLine(X, 32, X , 38, BLACK); display.update();} 
 
void Message_erreur_de_non_presence_capteurs() {BIP(); Aff_non_presence_capteurs(); delay (500);}
  
void Aff_non_presence_capteurs() {display.clear(); display.setCursor(20,32);
  Aff_TEXTE_EEPROM(243, 15); display.update();} 

/* >>> Lecture de la température et du taux d'humidité mesurés par un capteur DHT11 <<<
  * Broche_DATA : Broche sur laquelle est câblée la sortie du capteur.
  * Temperature : Pointeur vers la variable stockant la température.
  * Humidite : Pointeur vers la variable stockant le taux d'humidité.
  * Retourne DHT11_SUCCESS si aucune erreur, DHT11_TIMEOUT_ERROR en cas de dépassement,
    ou DHT11_CHECKSUM_ERROR en cas d'erreur de checksum */
  byte readDHT11(byte Broche_DATA, byte* Temperature, byte* Humidite) {
    // Variable_DATA[] : Tampon contenant les données du cpateur.
    // BIT_serie : Compteur permettant de savoir quel bit est reçu. (bitwise)
    // Index : Compteur permettant de savoir quel octet est reçu. (bitwise)
    // timeout : Compteur pour le délai en dépassement.
  byte Variable_DATA[5] = { 0 }, BIT_serie = 7, Index = 0;
  unsigned int timeout;
  // Conversion du numéro de broche Arduino en ports/masque binaire "bas niveau".
  // Utiliser les registres du microcontrôleur est bien plus rapide que digitalWrite().
  uint8_t bit = digitalPinToBitMask(Broche_DATA);
  uint8_t port = digitalPinToPort(Broche_DATA);
  volatile uint8_t *ddr = portModeRegister(port);   // Registre MODE (INPUT / OUTPUT)
  volatile uint8_t *out = portOutputRegister(port); // Registre OUT (Écriture)
  volatile uint8_t *in = portInputRegister(port);   // Registre IN (Lecture)
  // >>> Réveil du capteur <<<
  *ddr |= bit; // OUTPUT.
  *out &= ~bit; // LOW.
  delay(18); // Temps d'attente à LOW engendrant le réveil du capteur.
  *out |= bit; // HIGH
  delayMicroseconds(40);
  *ddr &= ~bit; // INPUT
  // >>> Attente de la réponse du capteur <<<
  timeout = 0;
  while(!(*in & bit)) // Attente d'un état LOW.
  if (++timeout == 10000)
  return DHT11_TIMEOUT_ERROR;
  timeout = 0;
  while(*in & bit) // Attente d'un état HIGH.
  if (++timeout == 10000)
  return DHT11_TIMEOUT_ERROR;
  // Lecture des 40 bits de données du capteur.
  for (byte i = 0; i < 40; ++i) {
    // Attente d'un état LOW.
    timeout = 0;
    while(!(*in & bit))
    if (++timeout == 10000)
    return DHT11_TIMEOUT_ERROR;
    // Mise en mémoire du "temps courant".
    unsigned long t = micros();
    // Attente d'un état HIGH.
    timeout = 0;    while(*in & bit)
    if (++timeout == 10000)
    return DHT11_TIMEOUT_ERROR;
    // Si le "delta Temps" est supérieur à 40µS c'est un "1", sinon c'est un "0".
    if ((micros() - t) > 40)
    Variable_DATA[Index] |= (1 << BIT_serie); // "1"
    // Variable_DATA[] est initialisé à "0" par défaut si le compteur de bits atteint zéro.
    if (BIT_serie-- == 0) {
      BIT_serie = 7; // On passe à l'octet suivant.
      ++Index; } }
  /* Format des données :
   * [0] = Humidité en %
   * [1] = zéro
   * [2] = Température en degrés Celsius
   * [3] = zéro
   * [4] = checksum (humidité + température)   */
  *Humidite = Variable_DATA[0];
  *Temperature = Variable_DATA[2];
  // >>> Vérifie la totalisation de contrôle. (Checksum) <<<
  if (Variable_DATA[4] != (Variable_DATA[0] + Variable_DATA[2]))
  return DHT11_CHECKSUM_ERROR;
  // Erreur de checksum.
  else return DHT11_SUCCESS; } // Pas d'erreur.
  
void Graduations_JOUR_SEMAINE() {
  if ((Delai_entre_Enregistrements == 5040) || (Delai_entre_Enregistrements == 20160))
     // Graduations pour une semaine ou 28 jours.
     {for (byte I=18; I < 120; I=I+17) display.drawLine(I, 19, I, 22, WHITE);}
  if (Delai_entre_Enregistrements == 20160) // Graduations pour 28 jours.
     {for (byte I=10; I < 120; I=I+17) display.drawLine(I, 19, I, 20, WHITE);}
  if (Delai_entre_Enregistrements == 720) // Graduations pour une journée.
     {for (byte I=9; I < 120; I=I+10) display.drawLine(I, 19, I, 22, WHITE);} }
     
void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); delay(100); noTone(BUZZER); pinMode(BUZZER, INPUT);}
  
void Aff_TEXTE_EEPROM(int PTR, byte Longueur) {
  for (byte I=0; I<Longueur; I++) {display.print(char(EEPROM.read(PTR))); PTR++;}; }
  
void Efface_interieur_cadre_jaune() {
  for (byte Y=1; Y < 15; Y++) display.drawLine(1, Y, 126 , Y, BLACK);}

void Affiche_cadre_du_haut(byte X) {
  CADRE_UP();  display.setTextSize(1); // Taille la plus petite.
  display.setCursor(X,4);}
  
void CADRE_UP() {display.drawRect(0, 0, 128, 16, WHITE);}
void CADRE() {display.drawRect(0, 16, 128, 48, WHITE);}

void ATTENDRE_un_BP() { Clic_sur_le_BP = false;
  while (!Clic_sur_le_BP) {
    digitalWrite(LED_Arduino,!digitalRead(LED_Arduino)); // Inverser la LED clavier.
    delay(50); Tester_le_BP_A0();}
  digitalWrite(LED_Arduino,LOW); Clic_sur_le_BP = false; }
  
void Affiche_1_espace() {display.print(" ");}
void Affiche_N_espaces(byte N) {for (byte I = N; I > 0; I--) Affiche_1_espace();}

void Effacer_zone(byte X, byte Y, byte NB_caracteres) {
  byte I = 10; 
  for (byte J=0; J < I; J++)
       display.drawLine(X, Y + J, X + (NB_caracteres * 6), Y + J, BLACK);}

// >>>>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  boolean Sauve_etat_LED_jaune;
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Sauve_etat_LED_jaune = digitalRead(LED_Jaune); Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Arduino, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    delay(100); // Attendre_la_stabilisation du travail.
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      if ((millis() - Horloge_actuelle) > 800) {Clic_long = true; digitalWrite(LED_Jaune, HIGH);}
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) COMPTEUR = 0; }
  digitalWrite(LED_Arduino, LOW);} // Éteindre la LED clavier.
  digitalWrite(LED_Jaune, Sauve_etat_LED_jaune); } // Restitue l'état de la LED jaune.
