/*  Test de la longue ligne de dialogue pour la NANO station MÉTÉO.
    Ce programme génère des signaux carrés sur la ligne pour observer à l'oscilloscope. */
    
// Déclarations pour l'afficheur OLED :
#include <Adafruit_ssd1306syp.h>
#define broche_SDA 11 // D11 utilisée en sortie.
#define broche_SCL 12 // D12 utilisée en sortie.
Adafruit_ssd1306syp display(broche_SDA, broche_SCL);

// Déclarations pour le capteur de pression BMP085.
// Mesure la pression de 300hPa à 1100hPa (soit de 9000m d’altitude à 500 sous le niveau de la mer)
// Lecture de la température entre -40 et 85°C (précision maximale entre 0 et 60°C)
// Résolution de 0.01 hPa et 0.1°C
// Précision de 0.2 à 0.5 hPa et de 0.5 à 1°C (en fonction de la température)
#include <Wire.h>
#include <Adafruit_BMP085.h> // Utilisation du capteur de pression atmosphérique BMP085.
  // BRANCHEMENTS imposés par la bibliothèque :
  // SLC est reliée à  l'entrée Analogique A5.
  // SDA est reliée à  l'entrée Analogique A4.
Adafruit_BMP085 Capteur_BMP085;

// Déclarations pour le capteur d'humidité DHT11.
#include <DHT11.h>
#define Pilotage_DH11 5
// Codes d'erreurs de la fonction readDHT11() :
enum {
  DHT11_SUCCESS,          // 0 : Pas d'erreur.
  DHT11_TIMEOUT_ERROR,    // 1 : Temps d'attente dépassé.
  DHT11_CHECKSUM_ERROR};  // 2 : Données reçues erronées.

// Constantes du programme.
#define Sentinelle 4  // D4 utilisée en sortie de pilotage à distance.
#define BUZZER  10    // Sortie dédiée au BUZZER.
#define SDA  18    // SDA est reliée à  l'entrée Analogique A4.
#define SCL  19    // SLC est reliée à  l'entrée Analogique A5.

// Variables du programme.
byte Temp_Humid, Humidite;  // Variables pour le capteur d'humidité.
byte Temperature; // Variable pour le capteur de pression.

void setup() {
  pinMode(BUZZER, INPUT);
  pinMode(Sentinelle, OUTPUT); digitalWrite(Sentinelle, HIGH);
  display.initialize(); delay(300); display.clear(); display.setTextSize(1); // Taille la plus petite.
  // ---------- Vérifier que la ligne dialogue correctement ----------
  // Cette vérification permet de s'assurer immédiatement que la ligne
  // n'a pas été inversée par exemple.
    Tester_les_capteurs(); display.clear();
    display.setCursor(12,20); display.print("TEST DE LA LIGNE");
    display.setCursor(12,32); display.print("de dialogue I2C.");
    display.update();
  pinMode(SDA, OUTPUT); pinMode(SLC, OUTPUT); pinMode(Pilotage_DH11, OUTPUT);}
  
void loop() {// Génération sans fin d'un signal carré à cadence maximale avec l'ATmega328.
  //------------------ Test n°1 de la ligne de pilotage de l'Alarme ----------------------
  digitalWrite(Sentinelle, HIGH); delayMicroseconds(1);
  digitalWrite(Sentinelle, LOW); delayMicroseconds(1);
  //----------------Test n°2 de la ligne SCL du capteur de pression BMP085 ---------------
//  digitalWrite(SCL, HIGH); delayMicroseconds(1);
//  digitalWrite(SCL, LOW); delayMicroseconds(1);
  //----------------Test n°3 de la ligne SDA du capteur de pression BMP085 ---------------
//  digitalWrite(SDA, HIGH); delayMicroseconds(1);
//  digitalWrite(SDA, LOW); delayMicroseconds(1);
  //--------------------- Test n°4 de la ligne de dialogue avec le DH11 ------------------
//  digitalWrite(Pilotage_DH11, HIGH); delayMicroseconds(1);
//  digitalWrite(Pilotage_DH11, LOW); delayMicroseconds(1);
  } // Pour pouvoir placer le test n°3 en remarque le délimiteur "}" est placé en dernière ligne.
 
void Tester_les_capteurs() {
  // Tant qu'aucun capteur d'humidité DHT11 n'est détecté boucle en ERREUR.
  while (readDHT11(Pilotage_DH11, &Temp_Humid, &Humidite) != DHT11_SUCCESS)
         Message_erreur_de_non_presence_capteurs();
  // Tant qu'aucun capteur de pression BMP085 n'est détecté boucle en ERREUR.
  while (!Capteur_BMP085.begin()) Message_erreur_de_non_presence_capteurs(); } 
 
void Message_erreur_de_non_presence_capteurs() {BIP(); display.clear(); display.setCursor(20,32);
  display.print("Capteurs NON OK"); display.update(); delay (500);}
  
void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); delay(100); noTone(BUZZER); pinMode(BUZZER, INPUT);}
  
/* >>> Lecture de la température et du taux d'humidité mesurés par un capteur DHT11 <<<
  * Broche_DATA : Broche sur laquelle est câblée la sortie du capteur.
  * Temperature : Pointeur vers la variable stockant la température.
  * Humidite : Pointeur vers la variable stockant le taux d'humidité.
  * Retourne DHT11_SUCCESS si aucune erreur, DHT11_TIMEOUT_ERROR en cas de dépassement,
    ou DHT11_CHECKSUM_ERROR en cas d'erreur de checksum */
  byte readDHT11(byte Broche_DATA, byte* Temperature, byte* Humidite) {
    // Variable_DATA[] : Tampon contenant les données du cpateur.
    // BIT_serie : Compteur permettant de savoir quel bit est reçu. (bitwise)
    // Index : Compteur permettant de savoir quel octet est reçu. (bitwise)
    // timeout : Compteur pour le délai en dépassement.
  byte Variable_DATA[5] = { 0 }, BIT_serie = 7, Index = 0;
  unsigned int timeout;
  // Conversion du numéro de broche Arduino en ports/masque binaire "bas niveau".
  // Utiliser les registres du microcontrôleur est bien plus rapide que digitalWrite().
  uint8_t bit = digitalPinToBitMask(Broche_DATA);
  uint8_t port = digitalPinToPort(Broche_DATA);
  volatile uint8_t *ddr = portModeRegister(port);   // Registre MODE (INPUT / OUTPUT)
  volatile uint8_t *out = portOutputRegister(port); // Registre OUT (Écriture)
  volatile uint8_t *in = portInputRegister(port);   // Registre IN (Lecture)
  // >>> Réveil du capteur <<<
  *ddr |= bit; // OUTPUT.
  *out &= ~bit; // LOW.
  delay(18); // Temps d'attente à LOW engendrant le réveil du capteur.
  *out |= bit; // HIGH
  delayMicroseconds(40);
  *ddr &= ~bit; // INPUT
  // >>> Attente de la réponse du capteur <<<
  timeout = 0;
  while(!(*in & bit)) // Attente d'un état LOW.
  if (++timeout == 10000)
  return DHT11_TIMEOUT_ERROR;
  timeout = 0;
  while(*in & bit) // Attente d'un état HIGH.
  if (++timeout == 10000)
  return DHT11_TIMEOUT_ERROR;
  // Lecture des 40 bits de données du capteur.
  for (byte i = 0; i < 40; ++i) {
    // Attente d'un état LOW.
    timeout = 0;
    while(!(*in & bit))
    if (++timeout == 10000)
    return DHT11_TIMEOUT_ERROR;
    // Mise en mémoire du "temps courant".
    unsigned long t = micros();
    // Attente d'un état HIGH.
    timeout = 0;    while(*in & bit)
    if (++timeout == 10000)
    return DHT11_TIMEOUT_ERROR;
    // Si le "delta Temps" est supérieur à 40µS c'est un "1", sinon c'est un "0".
    if ((micros() - t) > 40)
    Variable_DATA[Index] |= (1 << BIT_serie); // "1"
    // Variable_DATA[] est initialisé à "0" par défaut si le compteur de bits atteint zéro.
    if (BIT_serie-- == 0) {
      BIT_serie = 7; // On passe à l'octet suivant.
      ++Index; } }
  /* Format des données :
   * [0] = Humidité en %
   * [1] = zéro
   * [2] = Température en degrés Celsius
   * [3] = zéro
   * [4] = checksum (humidité + température)   */
  *Humidite = Variable_DATA[0];
  *Temperature = Variable_DATA[2];
  // >>> Vérifie la totalisation de contrôle. (Checksum) <<<
  if (Variable_DATA[4] != (Variable_DATA[0] + Variable_DATA[2]))
  return DHT11_CHECKSUM_ERROR;
  // Erreur de checksum.
  else return DHT11_SUCCESS; } // Pas d'erreur.
  

